@@ -65,19 +65,18 @@ pub struct StartEventContent {
 
     /// Information about the related event.
     #[serde(rename = "m.relates_to")]
-    pub relation: Relation,
+    pub relates_to: Relation,
 }
 
 #[cfg(feature = "unstable-pre-spec")]
 impl StartEventContent {
     /// Creates a new `StartEventContent` with the given device ID, method and relation.
-    pub fn new(from_device: DeviceIdBox, method: StartMethod, relation: Relation) -> Self {
-        Self { from_device, method, relation }
+    pub fn new(from_device: DeviceIdBox, method: StartMethod, relates_to: Relation) -> Self {
+        Self { from_device, method, relates_to }
     }
 }
 
-/// An enum representing the different method specific
-/// *m.key.verification.start* content.
+/// An enum representing the different method specific *m.key.verification.start* content.
 #[derive(Clone, Debug, Deserialize, Serialize)]
 #[cfg_attr(not(feature = "unstable-exhaustive-types"), non_exhaustive)]
 #[serde(untagged)]
@@ -87,19 +86,22 @@ pub enum StartMethod {
 
     /// The *m.reciprocate.v1* verification method.
     ///
-    /// The spec entry for this method can be found [here][1]
+    /// The spec entry for this method can be found [here][1].
     ///
     /// [1]: https://spec.matrix.org/unstable/client-server-api/#mkeyverificationstartmreciprocatev1
     #[cfg(feature = "unstable-pre-spec")]
     ReciprocateV1(ReciprocateV1Content),
 
     /// Any unknown start method.
-    Custom(CustomContent),
+    #[doc(hidden)]
+    _Custom(_CustomContent),
 }
 
 /// Method specific content of a unknown key verification method.
+#[doc(hidden)]
 #[derive(Clone, Debug, Deserialize, Serialize)]
-pub struct CustomContent {
+#[allow(clippy::exhaustive_structs)]
+pub struct _CustomContent {
     /// The name of the method.
     pub method: String,
 
@@ -120,7 +122,7 @@ pub struct ReciprocateV1Content {
 
 #[cfg(feature = "unstable-pre-spec")]
 impl ReciprocateV1Content {
-    /// Create a new `MReciprocateV1Content` with the given shared secret.
+    /// Create a new `ReciprocateV1Content` with the given shared secret.
     ///
     /// The shared secret needs to come from the scanned QR code, encoded using
     /// unpadded base64.
@@ -155,8 +157,9 @@ pub struct SasV1Content {
     pub short_authentication_string: Vec<ShortAuthenticationString>,
 }
 
-/// Mandatory initial set of fields for creating an `MSasV1Content`.
+/// Mandatory initial set of fields for creating an `SasV1Content`.
 #[derive(Clone, Debug, Deserialize)]
+#[allow(clippy::exhaustive_structs)]
 pub struct SasV1ContentInit {
     /// The key agreement protocols the sending device understands.
     ///
@@ -180,7 +183,7 @@ pub struct SasV1ContentInit {
 }
 
 impl SasV1Content {
-    /// Create a new `MSasV1Content` with the given values.
+    /// Create a new `SasV1Content` with the given values.
     ///
     /// # Errors
     ///
@@ -199,7 +202,7 @@ impl SasV1Content {
 impl TryFrom<SasV1ContentInit> for SasV1Content {
     type Error = InvalidInput;
 
-    /// Creates a new `MSasV1Content` from the given init struct.
+    /// Creates a new `SasV1Content` from the given init struct.
     fn try_from(init: SasV1ContentInit) -> Result<Self, Self::Error> {
         if !init.key_agreement_protocols.contains(&KeyAgreementProtocol::Curve25519)
             && !init.key_agreement_protocols.contains(&KeyAgreementProtocol::Curve25519HkdfSha256)
@@ -257,12 +260,14 @@ mod tests {
     };
 
     use super::{
-        CustomContent, HashAlgorithm, KeyAgreementProtocol, MessageAuthenticationCode,
-        SasV1Content, SasV1ContentInit, ShortAuthenticationString, StartMethod,
-        StartToDeviceEventContent,
+        HashAlgorithm, KeyAgreementProtocol, MessageAuthenticationCode, SasV1Content,
+        SasV1ContentInit, ShortAuthenticationString, StartMethod, StartToDeviceEventContent,
+        _CustomContent,
     };
     #[cfg(feature = "unstable-pre-spec")]
-    use super::{ReciprocateV1Content, Relation, StartEventContent};
+    use super::{ReciprocateV1Content, StartEventContent};
+    #[cfg(feature = "unstable-pre-spec")]
+    use crate::key::verification::Relation;
     use crate::ToDeviceEvent;
 
     #[test]
@@ -374,7 +379,7 @@ mod tests {
         let key_verification_start_content = StartToDeviceEventContent {
             from_device: "123".into(),
             transaction_id: "456".into(),
-            method: StartMethod::Custom(CustomContent {
+            method: StartMethod::_Custom(_CustomContent {
                 method: "m.sas.custom".to_owned(),
                 data: vec![("test".to_owned(), JsonValue::from("field"))]
                     .into_iter()
@@ -415,7 +420,7 @@ mod tests {
 
         let key_verification_start_content = StartEventContent {
             from_device: "123".into(),
-            relation: Relation { event_id: event_id.clone() },
+            relates_to: Relation { event_id: event_id.clone() },
             method: StartMethod::SasV1(
                 SasV1Content::new(SasV1ContentInit {
                     hashes: vec![HashAlgorithm::Sha256],
@@ -446,7 +451,7 @@ mod tests {
 
         let key_verification_start_content = StartEventContent {
             from_device: "123".into(),
-            relation: Relation { event_id: event_id.clone() },
+            relates_to: Relation { event_id: event_id.clone() },
             method: StartMethod::ReciprocateV1(ReciprocateV1Content::new(secret.clone())),
         };
 
@@ -563,7 +568,7 @@ mod tests {
                 content: StartToDeviceEventContent {
                     from_device,
                     transaction_id,
-                    method: StartMethod::Custom(CustomContent { method, data })
+                    method: StartMethod::_Custom(_CustomContent { method, data })
                 }
             } if from_device == "123"
                 && sender == user_id!("@example:localhost")
@@ -631,7 +636,7 @@ mod tests {
                 .unwrap(),
             StartEventContent {
                 from_device,
-                relation: Relation { event_id },
+                relates_to: Relation { event_id },
                 method: StartMethod::SasV1(SasV1Content {
                     hashes,
                     key_agreement_protocols,
@@ -663,7 +668,7 @@ mod tests {
                 .unwrap(),
             StartEventContent {
                 from_device,
-                relation: Relation { event_id },
+                relates_to: Relation { event_id },
                 method: StartMethod::ReciprocateV1(ReciprocateV1Content { secret }),
             } if from_device == "123"
                 && event_id == id
