@@ -5,13 +5,54 @@ use std::collections::BTreeMap;
 use js_int::UInt;
 use ruma_events_macros::EventContent;
 use ruma_identifiers::DeviceIdBox;
+#[cfg(feature = "unstable-pre-spec")]
+use ruma_identifiers::EventId;
 use serde::{Deserialize, Serialize};
 
-use crate::{room::message::Relation, MessageEvent};
+#[cfg(feature = "unstable-pre-spec")]
+use crate::{key::verification, reaction, room::message::Replacement};
+use crate::{
+    room::message::{self, InReplyTo},
+    MessageEvent,
+};
+
+mod relation_serde;
 
 /// An event that has been encrypted.
 pub type EncryptedEvent = MessageEvent<EncryptedEventContent>;
 
+/// The content payload for `EncryptedEvent`.
+#[derive(Clone, Debug, Deserialize, Serialize, EventContent)]
+#[cfg_attr(not(feature = "unstable-exhaustive-types"), non_exhaustive)]
+#[ruma_event(type = "m.room.encrypted", kind = Message, kind = ToDevice)]
+pub struct EncryptedEventContent {
+    /// Encrypted event content
+    #[serde(flatten)]
+    pub scheme: EncryptedEventScheme,
+
+    /// Information about related messages for [rich replies].
+    ///
+    /// [rich replies]: https://matrix.org/docs/spec/client_server/r0.6.1#rich-replies
+    #[serde(flatten, with = "relation_serde", skip_serializing_if = "Option::is_none")]
+    pub relates_to: Option<Relation>,
+}
+
+/// The to-device version of the payload for the `EncryptedEvent`.
+pub type EncryptedToDeviceEventContent = EncryptedEventContent;
+
+impl EncryptedEventContent {
+    /// Creates a new `EncryptedEventContent` with the given scheme and relation.
+    pub fn new(scheme: EncryptedEventScheme, relates_to: Option<Relation>) -> Self {
+        Self { scheme, relates_to }
+    }
+}
+
+impl From<EncryptedEventScheme> for EncryptedEventContent {
+    fn from(scheme: EncryptedEventScheme) -> Self {
+        Self { scheme, relates_to: None }
+    }
+}
+
 /// The encryption scheme for `EncryptedEventContent`
 #[derive(Clone, Debug, Deserialize, Serialize)]
 #[cfg_attr(not(feature = "unstable-exhaustive-types"), non_exhaustive)]
@@ -26,36 +67,72 @@ pub enum EncryptedEventScheme {
     MegolmV1AesSha2(MegolmV1AesSha2Content),
 }
 
-/// The content payload for `EncryptedEvent`.
-#[derive(Clone, Debug, Deserialize, Serialize, EventContent)]
+/// Relationship information about an encrypted event.
+///
+/// Outside of the encrypted payload to support server aggregation.
+#[derive(Clone, Debug)]
 #[cfg_attr(not(feature = "unstable-exhaustive-types"), non_exhaustive)]
-#[ruma_event(type = "m.room.encrypted", kind = Message, kind = ToDevice)]
-pub struct EncryptedEventContent {
-    /// Encrypted event content
-    #[serde(flatten)]
-    pub scheme: EncryptedEventScheme,
+pub enum Relation {
+    /// An `m.in_reply_to` relation indicating that the event is a reply to another event.
+    Reply {
+        /// Information about another message being replied to.
+        in_reply_to: InReplyTo,
+    },
 
-    /// Information about related messages for
-    /// [rich replies](https://matrix.org/docs/spec/client_server/r0.6.1#rich-replies).
-    #[serde(rename = "m.relates_to", skip_serializing_if = "Option::is_none")]
-    pub relates_to: Option<Relation>,
+    /// An event that replaces another event.
+    #[cfg(feature = "unstable-pre-spec")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "unstable-pre-spec")))]
+    Replacement(Replacement),
+
+    /// A reference to another event.
+    #[cfg(feature = "unstable-pre-spec")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "unstable-pre-spec")))]
+    Reference(Reference),
+
+    /// An annotation to an event.
+    #[cfg(feature = "unstable-pre-spec")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "unstable-pre-spec")))]
+    Annotation(Annotation),
 }
 
-impl EncryptedEventContent {
-    /// Creates a new `EncryptedEventContent` with the given scheme and relation.
-    pub fn new(scheme: EncryptedEventScheme, relates_to: Option<Relation>) -> Self {
-        Self { scheme, relates_to }
+/// A reference to another event.
+#[derive(Clone, Debug, Deserialize, Serialize)]
+#[cfg(feature = "unstable-pre-spec")]
+#[cfg_attr(docsrs, doc(cfg(feature = "unstable-pre-spec")))]
+#[cfg_attr(not(feature = "unstable-exhaustive-types"), non_exhaustive)]
+pub struct Reference {
+    /// The event we are referencing.
+    pub event_id: EventId,
+}
+
+#[cfg(feature = "unstable-pre-spec")]
+impl Reference {
+    /// Creates a new `Reference` with the given event ID.
+    pub fn new(event_id: EventId) -> Self {
+        Self { event_id }
     }
 }
 
-impl From<EncryptedEventScheme> for EncryptedEventContent {
-    fn from(scheme: EncryptedEventScheme) -> Self {
-        Self { scheme, relates_to: None }
-    }
+/// An annotation for an event.
+#[derive(Clone, Debug, Deserialize, Serialize)]
+#[cfg(feature = "unstable-pre-spec")]
+#[cfg_attr(docsrs, doc(cfg(feature = "unstable-pre-spec")))]
+#[cfg_attr(not(feature = "unstable-exhaustive-types"), non_exhaustive)]
+pub struct Annotation {
+    /// The event that is being annotated.
+    pub event_id: EventId,
+
+    /// The annotation.
+    pub key: String,
 }
 
-/// The to-device version of the payload for the `EncryptedEvent`.
-pub type EncryptedToDeviceEventContent = EncryptedEventContent;
+#[cfg(feature = "unstable-pre-spec")]
+impl Annotation {
+    /// Creates a new `Annotation` with the given event ID and key.
+    pub fn new(event_id: EventId, key: String) -> Self {
+        Self { event_id, key }
+    }
+}
 
 /// The payload for `EncryptedEvent` using the *m.olm.v1.curve25519-aes-sha2* algorithm.
 #[derive(Clone, Debug, Serialize, Deserialize)]
@@ -121,6 +198,7 @@ pub struct MegolmV1AesSha2Content {
 /// This struct will not be updated even if additional fields are added to `MegolmV1AesSha2Content`
 /// in a new (non-breaking) release of the Matrix specification.
 #[derive(Clone, Debug)]
+#[allow(clippy::exhaustive_structs)]
 pub struct MegolmV1AesSha2ContentInit {
     /// The encrypted content of the event.
     pub ciphertext: String,
@@ -143,14 +221,40 @@ impl From<MegolmV1AesSha2ContentInit> for MegolmV1AesSha2Content {
     }
 }
 
+impl From<message::Relation> for Relation {
+    fn from(rel: message::Relation) -> Self {
+        match rel {
+            message::Relation::Reply { in_reply_to } => Self::Reply { in_reply_to },
+            #[cfg(feature = "unstable-pre-spec")]
+            message::Relation::Replacement(re) => Self::Replacement(re),
+        }
+    }
+}
+
+#[cfg(feature = "unstable-pre-spec")]
+impl From<reaction::Relation> for Relation {
+    fn from(rel: reaction::Relation) -> Self {
+        let reaction::Relation { event_id, emoji } = rel;
+        Self::Annotation(Annotation { event_id, key: emoji })
+    }
+}
+
+#[cfg(feature = "unstable-pre-spec")]
+impl From<verification::Relation> for Relation {
+    fn from(rel: verification::Relation) -> Self {
+        let verification::Relation { event_id } = rel;
+        Self::Reference(Reference { event_id })
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use matches::assert_matches;
     use ruma_serde::Raw;
     use serde_json::{from_value as from_json_value, json, to_value as to_json_value};
 
-    use super::{EncryptedEventContent, EncryptedEventScheme, MegolmV1AesSha2Content};
-    use crate::room::message::{InReplyTo, Relation};
+    use super::{EncryptedEventContent, EncryptedEventScheme, MegolmV1AesSha2Content, Relation};
+    use crate::room::message::InReplyTo;
     use ruma_identifiers::event_id;
 
     #[test]
